 /********************************************************************/
 /*                                                                  */
 /*      Module:        PL/I Task Data Area (TDA)                    */
 /*      Version:       1.0                                          */
 /*      Date:          Jun, 2004                                    */
 /*      Author:        Peter Flass                                  */
 /*                                                                  */
 /*      Function:      TDA contains data describing one PL/I task.  */
 /*                                                                  */
 /*      Length:        96 bytes ('60'x).                            */
 /*                                                                  */
 /*      Where held:    Heap Storage.                                */
 /*                                                                  */
 /*      Pointed to by: Entry in 'GBL' points to the TDA for the     */
 /*                     major task.  TDAs are chained by             */
 /*                     attaching_task<->attached_task and in        */
 /*                     a global chain.                              */
 /*                                                                  */
 /*      Created by:    The TDA for the major task is created at     */
 /*                     startup.  Other TDAs are dynamically         */
 /*                     created by CALL statements with the TASK     */
 /*                     option.  New tasks are added to the front    */
 /*                     of the task chain, therefore the TDA for     */
 /*                     the major task will always be the last on    */
 /*                     the chain.                                   */
 /*                                                                  */
 /*      Deleted by:    Task termination.                            */
 /*                                                                  */
 /*      Dependencies:                                               */
 /*                                                                  */
 /*      To Do:                                                      */
 /*                                                                  */
 /*      Modifications:                                              */
 /*                                                                  */
 /********************************************************************/

 dcl   1 TDA                 aligned based,
         5 TDA_next          ptr,           /* addr(next_TDA)   '00'x*/
         5 TDA_prev          ptr,           /* addr(prev_TDA)   '04'x*/
         5 TDA_nextchild     ptr,           /* addr(next_child) '08'x*/
         5 TDA_prevchild     ptr,           /* addr(prev_child) '0C'x*/
         5 TDA_parent        ptr,           /* addr(parent)     '10'x*/
         5 TDA_TID           fixed bin(31), /* Thread ID        '14'x*/
         5 TDA_priority      fixed bin(31), /* Thread Priority  '18'x*/
         5 TDA_alloc_count   fixed bin(31), /* 'Count' at attach'1C'x*/
         5 TDA_event         ptr,           /* addr(event_var)  '20'x*/
         5 TDA_stack         ptr,           /* addr(thread stack'24'x*/
         5 TDA_heap          ptr,           /* addr(thread heap)'28'x*/
         5 TDA_flags         unaligned,     /* Thread Flags     '2C'x*/
         /* 1... .... Major Task thread ('80'bx) ( 1)                */
         5 fil1              char(1),       /* (alignment)      '2D'x*/
         5 TDA_level         fixed bin(15), /* Task depth       '2E'x*/
         5 TDA_ctl           ptr,           /* CTL stg chain    '30'x*/
         5 TDA_wait          ptr,           /* addr(evt) if wait'34'x*/
         5 fil2              char(40),      /* (future use)     '38'x*/
         5 TDA_end           char(0);       /* End address      '60'x*/

